from argparse import ArgumentParser


class Command:
    def __init__(self, base: ArgumentParser):
        self.parser = base
        self.parser.set_defaults(func=self.main)
        self.create()

    def main(self, args):
        raise NotImplementedError

    def create(self):
        raise NotImplementedError

    def parse(self):
        args = self.parser.parse_args()
        args.func(args)

    @classmethod
    def sub_command(cls, base: "_SubParsersAction") -> "Command":
        parser = base.add_parser(cls.__name__.lower())
        return cls(parser)



import argparse
from argparse import ArgumentParser

from .config import MeedConfig
from .command import Command

__version__ = "0.0.1"


class Base(Command):
    def __init__(self, parser):
        Command.__init__(self, parser)

    def main(self, args: argparse.Namespace):
        config = MeedConfig()
        if args.list:
            config.terminal_list_profiles()
        if args.new:
            config.terminal_create_new_profile()

    def create(self):
        self.parser.add_argument(
            "-v",
            "--version",
            action="version",
            version=__version__,
            help="Get the current meed version",
        )
        self.parser.add_argument(
            "-n",
            "--new",
            action="store_true",
            default=False,
            help="Setup a new meed profile",
        )
        self.parser.add_argument(
            "-l",
            "--list",
            action="store_true",
            default=False,
            help="Return a list of all meed profiles",
        )


if __name__ == "__main__":
    Base(
        ArgumentParser(
            description="A configurable dialogue for tracking reoccurring meed stats"
        )
    ).parse()



import argparse
from csv import DictWriter
from datetime import datetime
from sys import exit
from typing import *

import PySimpleGUI as sg

from .config import MeedConfig, MeedConfigNotExists
from .terminal import fatal_error
from .command import Command


class MeedLog:
    def __init__(self, config: MeedConfig):
        self.config = config
        sg.theme(self.config["Theme"])

        layout = self.radio_style()
        if self.config["Style"] == "Slider":
            layout = self.slider_style()
        elif self.config["Style"] == "Button":
            layout = self.button_style()

        self.window = sg.Window("Meed", layout)

        stats = self.extract_submit_event()
        now = datetime.now()
        stats["DateTime"] = now.strftime("%d-%m-%Y_%H:%M:%S")
        stats["Profile"] = self.config.profile_name
        self.write_to_csv(stats)

        self.window.close()
        return

    def write_to_csv(self, row: dict) -> None:
        headers = ["Profile", "DateTime", *self.config["Stats"]]
        with open(self.config["Output_File"], "a+", newline="") as file:
            csv_writer = DictWriter(file, fieldnames=headers)
            # Write headers if new file
            if file.tell() == 0:
                csv_writer.writeheader()
            csv_writer.writerow(row)

    def radio_style(self) -> list:
        layout = []
        for stat_group, stat in enumerate(self.config["Stats"]):
            layout.append([sg.Text(stat)])
            row = [
                sg.Radio(text=str(i), group_id=stat_group, key=f"{stat}_{i}")
                for i in range(1, self.config["Depth"] + 1)
            ]
            layout.append(row)
        layout.append([sg.Submit(), sg.Cancel()])
        return layout

    def slider_style(self) -> list:
        layout = []
        for stat_group, stat in enumerate(self.config["Stats"]):
            layout.append([sg.Text(stat)])
            layout.append([sg.Slider(range=(1, 10), orientation="horizontal")])
        layout.append([sg.Submit(), sg.Cancel()])
        return layout

    def button_style(self) -> list:
        layout = []
        for stat_group, stat in enumerate(self.config["Stats"]):
            layout.append([sg.Text(stat)])
            row = [
                sg.Button(
                    button_text=str(i),
                    button_type=sg.BUTTON_TYPE_REALTIME,
                    key=f"{stat}_{i}",
                )
                for i in range(1, self.config["Depth"] + 1)
            ]
            layout.append(row)
        layout.append([sg.Submit(), sg.Cancel()])
        return layout

    def extract_submit_event(self) -> Optional[dict]:
        while True:
            event, values = self.window.read()
            if event == "Submit":
                params = {}
                for key, val in values.items():
                    if val:
                        stat, stat_val = key.split("_")
                        params[stat] = stat_val
                if set(params.keys()) == set(self.config["Stats"]):
                    return params
                else:
                    sg.popup_error("Please select values for every stat", title="Error")
            elif event in [sg.WIN_CLOSED, "Cancel"]:
                self.window.close()
                exit()


class Log(Command):
    def __init__(self, parser):
        Command.__init__(self, parser)

    def main(self, args: argparse.Namespace):
        if args.log:
            try:
                config = MeedConfig(args.log)
                MeedLog(config)
            except MeedConfigNotExists:
                fatal_error(MeedConfigNotExists.MESSAGE)

    def create(self):
        self.parser.add_argument(
            "log",
            nargs="?",
            help="Name of the meed log profile. Use --new option to setup a profile",
        )


if __name__ == "__main__":
    Log(argparse.ArgumentParser()).parse()



import logging
from configparser import ConfigParser
from os import environ, path
from typing import *

from .terminal import header, required_property_input, property_input


class MeedConfigNotExists(FileNotFoundError):
    MESSAGE = "Could not load meed config, check configuration file exists and configuration profile is present"


class MeedConfig:
    REQUIRED_PROPERTIES = ["Output_File"]
    PROPERTIES = {
        "AWS_Profile": None,
        "Theme": "DarkGrey",
        "Stats": ["Happiness", "Confidence", "Frustration"],
        "Depth": 10,
        "Style": "Radio",
    }

    def __init__(self, profile: str or None = None):
        self.configuration_file_path = environ.get(
            "MEED_CONFIG_PATH",
            path.join(path.expanduser("~"), ".config", "meed", "config.ini"),
        )
        self.config = None
        self.profile_name = profile
        if profile:
            self.load_profile(profile)

    def load_profile(self, profile_name):
        config = ConfigParser()
        config.read(self.configuration_file_path)
        try:
            self.config = config[profile_name]
        except KeyError:
            logging.error(
                f"invalid profile '{profile_name}' for configuration '{self.configuration_file_path}'"
            )
            raise MeedConfigNotExists

    def terminal_list_profiles(self):
        header("Available meed profiles")
        for profile in self.get_profiles():
            print(" • " + profile)

    def terminal_create_new_profile(self):
        header("Setup new meed log profile")
        profile_name = required_property_input("Profile_Name")
        options = {
            prop: required_property_input(prop)
            for prop in MeedConfig.REQUIRED_PROPERTIES
        }
        for prop, default in MeedConfig.PROPERTIES.items():
            if type(default) is list:
                default = ",".join(default)
            val = property_input(prop, default)
            if type(default) is list and type(val) is str:
                options[prop] = val.strip()
            elif type(default) is int and type(val):
                options[prop] = str(val)
            elif val is None:
                continue
            else:
                options[prop] = val
        self._create_new_profile(profile_name=profile_name, options=options)

    def _create_new_profile(self, profile_name: str, options=None):
        if options is None:
            options = {}
        config = ConfigParser()
        config[profile_name] = {}
        for key, val in options.items():
            config[profile_name][key] = val

        with open(self.configuration_file_path, "a+") as config_file:
            config.write(config_file)
        self.config = config

    def get_profiles(self) -> List[str]:
        if path.exists(self.configuration_file_path):
            config = ConfigParser()
            config.read(self.configuration_file_path)
            return config.sections()
        return []

    def __getitem__(self, item: str) -> Any:
        if item in MeedConfig.REQUIRED_PROPERTIES:
            return self.config[item]
        if item in MeedConfig.PROPERTIES:
            val = self.config.get(item, MeedConfig.PROPERTIES[item])
            if type(MeedConfig.PROPERTIES[item]) is list:
                val = val.split(",")
            elif type(MeedConfig.PROPERTIES[item]) is int:
                val = int(val)
            return val



import typing

from colorama import Fore, Back, Style


def property_input(prompt: str, alt: typing.Any = None) -> typing.Optional[str]:
    inp = input(Back.WHITE + f"[{prompt}] ({alt}):" + Style.RESET_ALL + " ")
    if inp == "":
        return alt
    return inp


def required_property_input(prompt: str) -> str:
    def print_prompt() -> str:
        return input(
            f"{Back.WHITE}[{Fore.RED}*{Fore.RESET}{prompt}]:{Style.RESET_ALL} "
        )

    inp = print_prompt()
    while inp.strip() == "":
        inp = print_prompt()
    return inp


def header(heading: str) -> None:
    print(Back.LIGHTBLUE_EX + Style.BRIGHT + "   " + heading + "   " + Style.RESET_ALL)


def fatal_error(message: str) -> None:
    print(Back.RED + Style.BRIGHT + message + Style.RESET_ALL)


def sudden_exit():
    print("\nExiting meed!")



import argparse

from .command import Command


class Graph(Command):
    def __init__(self, parser):
        Command.__init__(self, parser)

    def main(self, args: argparse.Namespace):
        pass

    def create(self):
        pass


if __name__ == "__main__":
    Graph(argparse.ArgumentParser()).parse()



from argparse import ArgumentParser
from meedlib import Log
from meedlib import Base
from meedlib import Graph
from meedlib import Stat


def main():
    # Construct all sub-commands under one main base command
    base = Base(
        ArgumentParser(
            description="A configurable dialogue for tracking reoccurring mood stats"
        )
    )
    sub_parsers = base.parser.add_subparsers(help="Moody subcommands")

    Log.sub_command(sub_parsers)
    Graph.sub_command(sub_parsers)
    Stat.sub_command(sub_parsers)

    base.parse()


if __name__ == "__main__":
    main()



import argparse
from csv import reader

from .config import MeedConfig, MeedConfigNotExists
from .terminal import fatal_error
from .command import Command


class Stat(Command):
    def __init__(self, parser):
        Command.__init__(self, parser)

    def main(self, args: argparse.Namespace):
        if not args.log:
            return
        try:
            config = MeedConfig(args.log)
            relevant_stats = []
            with open(config["Output_File"]) as file:
                csv_reader = reader(file)
                for row in csv_reader:
                    profile, datetime, stats = row[0], row[1], row[2:]
                    if profile != args.log:
                        continue
                    relevant_stats.append([int(i) for i in stats])
            print(relevant_stats)
        except MeedConfigNotExists:
            fatal_error(MeedConfigNotExists.MESSAGE)
        except FileNotFoundError:
            fatal_error(
                f"Could not find output file, check your configuration"
            )

    def create(self):
        self.parser.add_argument(
            "log",
            help="Name of the meed log profile",
        )
        self.parser.add_argument(
            "--start_time",
            help="Number of days to calculate stats back to",
        )
        self.parser.add_argument(
            "--end_time",
            help="Number of days to calculate stats back to",
        )


if __name__ == "__main__":
    Stat(argparse.ArgumentParser()).parse()
